<?php
namespace Drupal\cleantalk;
/**
Cleantalk Spam FireWall class
**/

class CleantalkSFW
{
	public $ip = 0;
	public $ip_str = '';
	public $ip_array = Array();
	public $ip_str_array = Array();
	public $blocked_ip = '';
	public $passed_ip = '';
	public $result = false;

	public static $cdn_pool = array(
		'cloud_flare' => array(
			'103.21.244.0/22',
			'103.22.200.0/22',
			'103.31.4.0/22',
			'104.16.0.0/12',
			'108.162.192.0/18',
			'131.0.72.0/22',
			'141.101.64.0/18',
			'162.158.0.0/15',
			'172.64.0.0/13',
			'173.245.48.0/20',
			'188.114.96.0/20',
			'190.93.240.0/20',
			'197.234.240.0/22',
			'198.41.128.0/17',
		),
	);	
	public static $private_networks = array(
		'10.0.0.0/8',
		'100.64.0.0/10',
		'172.16.0.0/12',
		'192.168.0.0/16',
		'127.0.0.1/32',
	);	
	public function cleantalk_get_real_ip()
	{
		$headers = function_exists('apache_request_headers') ? apache_request_headers() : $this->apache_request_headers();

		$result['remote_addr'] = filter_var( $_SERVER['REMOTE_ADDR'], FILTER_VALIDATE_IP, FILTER_FLAG_IPV4 );
		
		// Cloud Flare
		if(isset($headers['Cf-Connecting-Ip'])){
			if($this->ip_mask_match($result['remote_addr'], self::$cdn_pool['cloud_flare'])){
				$result['cf_connecting_ip'] = filter_var( $headers['Cf-Connecting-Ip'], FILTER_VALIDATE_IP, FILTER_FLAG_IPV4 );
				unset($result['remote_addr']);
			}		
		// Private networks. Looking for HTTP_X_FORWARDED_FOR and HTTP_X_REAL_IP
		}elseif($this->ip_mask_match($result['remote_addr'], self::$private_networks)){
			if(isset($headers['X-Forwarded_For'])){
				$result['x_forwarded_for'] = filter_var( $headers['X-Forwarded_For'], FILTER_VALIDATE_IP, FILTER_FLAG_IPV4 );
			}
			if(isset($headers['X-Real-Ip'])){
				$result['x_real_ip'] = filter_var( $headers['X-Real-Ip'], FILTER_VALIDATE_IP, FILTER_FLAG_IPV4 );
			}
			if(isset($result['x_forwarded_for']) || isset($result['x_real_ip'])){
				unset($result['remote_addr']);
			}
		}
		
		if(isset($_GET['sfw_test_ip'])){
			$result['test'] = filter_var( $_GET['sfw_test_ip'], FILTER_VALIDATE_IP, FILTER_FLAG_IPV4 );
		}
		
		$this->ip_array = $result;

		return $result;
	}
	/*
	 * Check if the IP belong to mask. Recursivly if array given
	 * @param ip string  
	 * @param cird mixed (string|array of strings)
	*/
	public function ip_mask_match($ip, $cidr){
		if(is_array($cidr)){
			foreach($cidr as $curr_mask){
				if($this->ip_mask_match($ip, $curr_mask)){
					return true;
				}
			} unset($curr_mask);
			return false;
		}
		$exploded = explode ('/', $cidr);
		$net = $exploded[0];
		$mask = 4294967295 << (32 - $exploded[1]);
		return (ip2long($ip) & $mask) == (ip2long($net) & $mask);
	}	
	/**
	 * Function sends raw request to API server
	 *
	 * @param string url of API server
	 * @param array data to send
	 * @param boolean is data have to be JSON encoded or not
	 * @param integer connect timeout
	 * @return type
	 */

	public function sendRawRequest($url,$data,$isJSON=false,$timeout=3) {
	    $result=null;
	    if(!$isJSON) {
	        $data=http_build_query($data);
	    }
	    else {
	        $data= \Drupal\Component\Serialization\Json::encode($data);
	    }
	    $curl_exec=false;
	    if (function_exists('curl_init')) {
	    
	        $ch = curl_init();
	        curl_setopt($ch, CURLOPT_URL, $url);
	        curl_setopt($ch, CURLOPT_TIMEOUT, $timeout);
	        curl_setopt($ch, CURLOPT_POST, true);
	        curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
	        
	        // receive server response ...
	        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
	        // resolve 'Expect: 100-continue' issue
	        curl_setopt($ch, CURLOPT_HTTPHEADER, array('Expect:'));
	        
	        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
	        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
	        
	        $result = @curl_exec($ch);
	        if($result!==false) {
	            $curl_exec=true;
	        }
	        @curl_close($ch);
	    }
	    if(!$curl_exec) {
	        $opts = array(
	            'http'=>array(
	                'method'=>"POST",
	                'content'=>$data)
	        );
	        $context = stream_context_create($opts);
	        $result = @file_get_contents($url, 0, $context);
	    }
	    return $result;
	}	
	public function check_ip()
	{				
		$r = \Drupal::service('database')->query("SHOW TABLES LIKE '{cleantalk_sfw}'");
		$ob = $r->fetchObject();
		if(isset($ob) && is_object($ob)){
			foreach ($this->ip_array as $current_ip)
			{
				$r = \Drupal::service('database')->query("SELECT count(network) AS cnt FROM {cleantalk_sfw} WHERE network = :network & mask", array(':network' => sprintf("%u", ip2long($current_ip))));

					$cnt = intval($r->fetchObject()->cnt);
				if($cnt>0)
				{
					$this->result=true;
					$this->blocked_ip= $current_ip;
				}
				else
				{
					$this->passed_ip = $current_ip;
				}
			}
		}
		if($this->passed_ip != '')
		{
			$key = \Drupal::config('cleantalk.settings')->get('cleantalk_authkey');
			@setcookie ('apbct_sfw_pass_key', md5($this->passed_ip . $key), 0, "/");
		}
	}
	
	public function sfw_die()
	{
		$key=\Drupal::config('cleantalk.settings')->get('cleantalk_authkey');
		$sfw_die_page=file_get_contents(dirname(__FILE__)."/sfw_die_page.html");
		$sfw_die_page=str_replace("{REMOTE_ADDRESS}",$this->blocked_ip,$sfw_die_page);
		$sfw_die_page=str_replace("{REQUEST_URI}",$_SERVER['REQUEST_URI'],$sfw_die_page);
		$sfw_die_page=str_replace("{SFW_COOKIE}",md5($this->blocked_ip.$key),$sfw_die_page);
		@setcookie ('apbct_sfw_passed', '0');
		@header('HTTP/1.0 403 Forbidden');
		print $sfw_die_page;
		die();
	}
	
	static public function sfw_update($ct_key){		
		$data = Array();
		$method_name = '2s_blacklists_db';
		$result=self::sendRawRequest("https://api.cleantalk.org/?auth_key=$ct_key&method_name=$method_name",$data,false);
						
		$result=json_decode($result, true);
		
		if(isset($result['data'])){

			\Drupal::service('database')->query("TRUNCATE TABLE {cleantalk_sfw}");
			
			$result=$result['data'];
			$query="INSERT INTO {cleantalk_sfw} VALUES ";
			for($i=0;$i<sizeof($result);$i++){
				if($i==sizeof($result)-1){
					$query.="(".$result[$i][0].",".$result[$i][1].");";
				}else{
					$query.="(".$result[$i][0].",".$result[$i][1]."), ";
				}
			}
			$result = \Drupal::service('database')->query($query);
		}
	}
	
	//Add entries to SFW log
	static public function sfw_update_logs($ip, $result){
		if($ip === NULL || $result === NULL){
			error_log('SFW log update failed');
			return;
		}
		
		$blocked = ($result == 'blocked' ? ' + 1' : '');
		$time = time();

		$query = "INSERT INTO {cleantalk_sfw_logs}
		SET 
			`ip` = '$ip',
			`all_entries` = 1,
			`blocked_entries` = 1,
			`entries_timestamp` = '".$time."'
		ON DUPLICATE KEY 
		UPDATE 
			`all_entries` = `all_entries` + 1,
			`blocked_entries` = `blocked_entries`".$blocked.",
			`entries_timestamp` = '".$time."'";

		$result = \Drupal::service('database')->query($query);		
	}
	
	//Send and wipe SFW log
	public static function send_logs($ct_key){
		
		//Getting logs		
		$result = db_select('{cleantalk_sfw_logs}', 'ct_sfw')
			->fields('ct_sfw')
			->execute()
			->fetchAll();
				
		if(!empty($result)){
			
			// Compile logs
			$data = array();
			foreach($result as $key => $value){
				$data[] = array(trim($value->ip), (int)$value->all_entries, (int)($value->all_entries - $value->blocked_entries), (int)$value->entries_timestamp);
			} unset($key, $value, $result);
			
			//Final compile
			$qdata = array (
				'data' => json_encode($data),
				'rows' => count($data),
				'timestamp' => time()
			);
			
			//Sendings request
			$result=self::sendRawRequest('https://api.cleantalk.org/?method_name=sfw_logs&auth_key='.$ct_key, $qdata, false);
			
			$result = json_decode($result);
			//Checking answer and truncate table, reset counter
			if(isset($result->data) && isset($result->data->rows))
				if($result->data->rows == count($data)){
					\Drupal::service('database')->query("TRUNCATE TABLE {cleantalk_sfw_logs}");
				}
		}
	}
	public function apache_request_headers(){
		
		$headers = array();	
		foreach($_SERVER as $key => $val){
			if(preg_match('/\AHTTP_/', $key)){
				$server_key = preg_replace('/\AHTTP_/', '', $key);
				$key_parts = explode('_', $server_key);
				if(count($key_parts) > 0 and strlen($server_key) > 2){
					foreach($key_parts as $part_index => $part){
						$key_parts[$part_index] = function_exists('mb_strtolower') ? mb_strtolower($part) : strtolower($part);
						$key_parts[$part_index][0] = strtoupper($key_parts[$part_index][0]);					
					}
					$server_key = implode('-', $key_parts);
				}
				$headers[$server_key] = $val;
			}
		}
		return $headers;
	}	
}